# -*- coding: utf-8 -*-

"""
This file is part of the Ingram Micro Cloud Blue Connect SDK.
Copyright (c) 2019 Ingram Micro. All Rights Reserved.
"""

from .fulfillment_automation import FulfillmentAutomation
from .automation import AutomationResource
from .template import TemplateResource
from .tier_config_automation import TierConfigAutomation
from .usage_automation import UsageAutomation
from .usage_file_automation import UsageFileAutomation


__all__ = [
    'FulfillmentAutomation',
    'AutomationResource',
    'TemplateResource',
    'TierConfigAutomation',
    'UsageAutomation',
    'UsageFileAutomation',
]



# -*- coding: utf-8 -*-

"""
This file is part of the Ingram Micro Cloud Blue Connect SDK.
Copyright (c) 2019 Ingram Micro. All Rights Reserved.
"""

from marshmallow import fields, post_load
from typing import List, Optional

from .base import BaseModel, BaseSchema
from .connection import Connection, ConnectionSchema
from .parameters import Param, ParamSchema
from .product import Item, ItemSchema, Product, ProductSchema
from .tiers import Tiers, TiersSchema


class Asset(BaseModel):
    status = None  # type: str
    external_id = None  # type: str
    external_uid = None  # type: str
    product = None  # type: Product
    connection = None  # type: Connection
    items = None  # type: List[Item]
    params = None  # type: List[Param]
    tiers = None  # type: Tiers

    def get_param_by_id(self, id_):
        # type: (str) -> Optional[Param]
        try:
            return list(filter(lambda param: param.id == id_, self.params))[0]
        except IndexError:
            return None

    def get_item_by_mpn(self, mpn):
        # type: (str) -> Optional[Item]
        try:
            return list(filter(lambda item: item.mpn == mpn, self.items))[0]
        except IndexError:
            return None


class AssetSchema(BaseSchema):
    status = fields.Str()
    external_id = fields.Str()
    external_uid = fields.UUID()
    product = fields.Nested(ProductSchema, only=('id', 'name'))
    connection = fields.Nested(
        ConnectionSchema, only=('id', 'type', 'provider', 'vendor'),
    )
    items = fields.List(fields.Nested(ItemSchema))
    params = fields.List(fields.Nested(ParamSchema))
    tiers = fields.Nested(TiersSchema)

    @post_load
    def make_object(self, data):
        return Asset(**data)



# -*- coding: utf-8 -*-

"""
This file is part of the Ingram Micro Cloud Blue Connect SDK.
Copyright (c) 2019 Ingram Micro. All Rights Reserved.
"""

from marshmallow import fields, post_load

from .base import BaseModel, BaseSchema


class Company(BaseModel):
    name = None  # type: str


class CompanySchema(BaseSchema):
    name = fields.Str()

    @post_load
    def make_object(self, data):
        return Company(**data)



# -*- coding: utf-8 -*-

"""
This file is part of the Ingram Micro Cloud Blue Connect SDK.
Copyright (c) 2019 Ingram Micro. All Rights Reserved.
"""

from marshmallow import fields, post_load

from .base import BaseModel, BaseSchema


class Product(BaseModel):
    name = None  # type: str
    icon = None  # type: str


class ProductSchema(BaseSchema):
    name = fields.Str()
    icon = fields.Str()

    @post_load
    def make_object(self, data):
        return Product(**data)


class Item(BaseModel):
    global_id = None  # type: str
    mpn = None  # type: str
    old_quantity = None  # type: int
    quantity = None  # type: int


class ItemSchema(BaseSchema):
    global_id = fields.Str()
    mpn = fields.Str()
    old_quantity = fields.Integer()
    quantity = fields.Integer()

    @post_load
    def make_object(self, data):
        return Item(**data)



# -*- coding: utf-8 -*-

"""
This file is part of the Ingram Micro Cloud Blue Connect SDK.
Copyright (c) 2019 Ingram Micro. All Rights Reserved.
"""

from marshmallow import fields, post_load

from .asset import Asset, AssetSchema
from .base import BaseModel, BaseSchema
from .marketplace import Contract, ContractSchema, Marketplace, MarketplaceSchema


class Fulfillment(BaseModel):
    activation_key = None  # type: str
    asset = None  # type: Asset
    status = None  # type: str
    type = None  # type: str
    updated = None  # type: str
    created = None  # type: str
    reason = None  # type: str
    params_from_url = None  # type: str
    contract = None  # type: Contract
    marketplace = None  # type: Marketplace

    @property
    def new_items(self):
        # noinspection PyUnresolvedReferences
        return list(filter(
            lambda item: item.quantity > 0 and item.old_quantity == 0,
            self.asset.items))

    @property
    def changed_items(self):
        # noinspection PyUnresolvedReferences
        return list(filter(
            lambda item: item.quantity > 0 and item.old_quantity > 0,
            self.asset.items))

    @property
    def removed_items(self):
        # noinspection PyUnresolvedReferences
        return list(filter(
            lambda item: item.quantity == 0 and item.old_quantity > 0,
            self.asset.items))


class FulfillmentSchema(BaseSchema):
    activation_key = fields.Str()
    asset = fields.Nested(AssetSchema)
    status = fields.Str()
    type = fields.Str()
    updated = fields.DateTime()
    created = fields.DateTime()
    reason = fields.Str()
    params_form_url = fields.Str()
    contract = fields.Nested(ContractSchema, only=('id', 'name'))
    marketplace = fields.Nested(MarketplaceSchema, only=('id', 'name'))

    @post_load
    def make_object(self, data):
        return Fulfillment(**data)



# -*- coding: utf-8 -*-

"""
This file is part of the Ingram Micro Cloud Blue Connect SDK.
Copyright (c) 2019 Ingram Micro. All Rights Reserved.
"""
from marshmallow import fields, post_load

from connect.models.base import BaseModel, BaseSchema
from connect.models.company import Company, CompanySchema
from connect.models.marketplace import Contract, Marketplace, ContractSchema, MarketplaceSchema
from connect.models.product import Product, ProductSchema


class Records(BaseModel):
    valid = None  # type: int
    invalid = None  # type: int


class File(BaseModel):
    name = None  # type: str
    description = None  # type: str
    note = None  # type: str
    status = None  # type: str
    created_by = None  # type: str
    created_at = None  # type: str
    product = None  # type: Product
    contract = None  # type: Contract
    marketplace = None  # type: Marketplace
    vendor = None  # type: Company
    provider = None  # type: Company
    upload_file_uri = None  # type: str
    processed_file_uri = None  # type: str
    acceptance_note = None  # type: str
    rejection_note = None  # type: str
    error_detail = None  # type: str
    records = None  # type: Records
    uploaded_by = None  # type: str
    uploaded_at = None  # type: str
    submitted_by = None  # type: str
    submitted_at = None  # type: str
    accepted_by = None  # type: str
    accepted_at = None  # type: str
    rejected_by = None  # type: str
    rejected_at = None  # type: str
    closed_by = None  # type: str
    closed_at = None  # type: str


class Listing(BaseModel):
    status = None  # type: str
    contract = None  # type: Contract
    product = None  # type: Product
    created = None  # type: str

    # Undocumented fields (they appear in PHP SDK)
    vendor = None  # type: Company
    provider = None  # type: Company


class FileUsageRecord(BaseModel):
    record_id = None  # type: str
    item_search_criteria = None  # type: str
    item_search_value = None  # type: str
    quantity = None  # type: int
    start_time_utc = None  # type: str
    end_time_utc = None  # type: str
    asset_search_criteria = None  # type: str
    asset_search_value = None  # type: str


class RecordsSchema(BaseSchema):
    valid = fields.Int()
    invalid = fields.Int()

    @post_load
    def make_object(self, data):
        return Records(**data)


class FileSchema(BaseSchema):
    name = fields.Str()
    description = fields.Str()
    note = fields.Str()
    status = fields.Str()
    created_by = fields.Str()
    created_at = fields.Str()
    product = fields.Nested(ProductSchema)
    contract = fields.Nested(ContractSchema)
    marketplace = fields.Nested(MarketplaceSchema)
    vendor = fields.Nested(CompanySchema)
    provider = fields.Nested(CompanySchema)
    upload_file_uri = fields.Str()
    processed_file_uri = fields.Str()
    acceptance_note = fields.Str()
    rejection_note = fields.Str()
    error_detail = fields.Str()
    records = fields.Nested(RecordsSchema)
    uploaded_by = fields.Str()
    uploaded_at = fields.Str()
    submitted_by = fields.Str()
    submitted_at = fields.Str()
    accepted_by = fields.Str()
    accepted_at = fields.Str()
    rejected_by = fields.Str()
    rejected_at = fields.Str()
    closed_by = fields.Str()
    closed_at = fields.Str()

    @post_load
    def make_object(self, data):
        return File(**data)


class ListingSchema(BaseSchema):
    status = fields.Str()
    contract = fields.Nested(ContractSchema)
    product = fields.Nested(ProductSchema)
    created = fields.Str()

    # Undocumented fields (they appear in PHP SDK)
    vendor = fields.Nested(CompanySchema)
    provider = fields.Nested(CompanySchema)

    @post_load
    def make_object(self, data):
        return Listing(**data)


class FileUsageRecordSchema(BaseSchema):
    record_id = fields.Str()
    item_search_criteria = fields.Str()
    item_search_value = fields.Str()
    quantity = fields.Int()
    start_time_utc = fields.Str()
    end_time_utc = fields.Str()
    asset_search_criteria = fields.Str()
    asset_search_value = fields.Str()

    @post_load
    def make_object(self, data):
        return FileUsageRecord(**data)



# -*- coding: utf-8 -*-

"""
This file is part of the Ingram Micro Cloud Blue Connect SDK.
Copyright (c) 2019 Ingram Micro. All Rights Reserved.
"""

from marshmallow import Schema, fields, post_load

from .base import BaseModel, BaseSchema


class Hub(BaseModel):
    name = None  # type: str


class HubSchema(BaseSchema):
    name = fields.Str()

    @post_load
    def make_object(self, data):
        return Hub(**data)


class Hubs(BaseModel):
    hub = None  # type: Hub
    external_id = None  # type: str


class HubsSchema(Schema):
    hub = fields.Nested(HubSchema, only=('id', 'name'))
    external_id = fields.Str()

    @post_load
    def make_object(self, data):
        return Hubs(**data)



# -*- coding: utf-8 -*-

"""
This file is part of the Ingram Micro Cloud Blue Connect SDK.
Copyright (c) 2019 Ingram Micro. All Rights Reserved.
"""
import functools
from typing import Any, List, Dict, Union

import marshmallow
import requests
from requests import compat

from connect.config import Config
from connect.logger import function_log, logger
from connect.models import BaseSchema, ServerErrorSchema
from connect.models.exception import ServerErrorException


class ApiClient(object):
    def __init__(self, config, base_path):
        # type: (Config, str) -> None

        # Assign base URL
        self._base_path = base_path

        # Assign passed config or globally configured instance
        self._config = config or Config.get_instance()

    @property
    def base_path(self):
        # type: () -> str
        return self._base_path

    @property
    def config(self):
        # type: () -> Config
        return self._config

    @property
    def headers(self):
        # type: () -> Dict[str, str]
        return {
            'Authorization': (self.config.api_key
                              if self.config.api_key.startswith('ApiKey ')
                              else 'ApiKey ' + self.config.api_key),
            'Content-Type': 'application/json',
        }

    def get_url(self, path=''):
        # type: (str) -> str
        return self._urljoin(self.config.api_url, self.base_path, path)

    @function_log
    def get(self, path='', params=None, **kwargs):
        kwargs = self._fix_request_kwargs(kwargs, path, params=params)
        response = requests.get(**kwargs)
        return self._check_response(response)

    @function_log
    def post(self, path='', data=None, json=None, **kwargs):
        kwargs = self._fix_request_kwargs(kwargs, path, data=data, json=json)
        response = requests.post(**kwargs)
        return self._check_response(response)

    @function_log
    def put(self, path='', data=None, **kwargs):
        kwargs = self._fix_request_kwargs(kwargs, path, data=data)
        response = requests.put(**kwargs)
        return self._check_response(response)

    def _fix_request_kwargs(self, prev_kwargs, path, **kwargs):
        # type: (Dict[str, Any], str, Dict[str, Any]) -> Dict[str, Any]
        """ Set correct kwargs for requests """
        fixed_kwargs = prev_kwargs.copy()
        fixed_kwargs.update(kwargs)
        if 'url' not in fixed_kwargs:
            fixed_kwargs['url'] = self.get_url(path)
        if 'headers' not in fixed_kwargs:
            fixed_kwargs['headers'] = self.headers
        return fixed_kwargs

    @staticmethod
    def _urljoin(*args):
        return functools.reduce(
            lambda a, b: compat.urljoin(a + ('' if a.endswith('/') else '/'), b),
            args)

    @staticmethod
    def _check_response(response):
        # type: (requests.Response) -> str
        if not hasattr(response, 'content'):
            raise AttributeError(
                'Response does not have attribute content. Check your request params. '
                'Response status - {}'.format(response.status_code),
            )

        if not hasattr(response, 'ok') or not response.ok:
            data, error = ServerErrorSchema().loads(response.content)
            if data:
                raise ServerErrorException(data)

        return response.content


class BaseResource(object):
    resource = None  # type: str
    limit = 100  # type: int
    schema = BaseSchema()  # type: marshmallow.Schema

    def __init__(self, config=None):
        # Set api
        if not self.__class__.resource:
            raise AttributeError('Resource name not specified in class {}. '
                                 .format(self.__class__.__name__) +
                                 'Add an attribute `resource` name of the resource')
        self._api = ApiClient(config, self.__class__.resource)

        # Set passed config or globally configured instance
        self._config = config or Config.get_instance()
        if not isinstance(self.config, Config):
            raise ValueError('A valid Config object must be passed or globally configured '
                             'to create a ' + type(self).__name__)

    @property
    def api(self):
        # type: () -> ApiClient
        return self._api

    @property
    def config(self):
        # type: () -> Config
        return self._config

    @property
    def list(self):
        # type: () -> List[Any]
        filters = self.build_filter()
        logger.info('Get list request by filter - {}'.format(filters))
        response = self.api.get(params=filters)
        return self._load_schema(response)

    def build_filter(self):
        # type: () -> Dict[str, Any]
        res_filter = {}
        if self.limit:
            res_filter['limit'] = self.limit
        return res_filter

    def get(self, pk):
        # type: (str) -> Any
        response = self.api.get(path=pk)
        objects = self._load_schema(response)
        if isinstance(objects, list) and len(objects) > 0:
            return objects[0]

    def _load_schema(self, response, many=None):
        # type: (str, bool) -> Union[List[Any], Any]
        objects, error = self.schema.loads(response, many)
        if error:
            raise TypeError(
                'Invalid structure for initialization of `{}`. \n'
                'Error: {}. \nServer Response: {}'.format(type(self).__name__, error, response),
            )

        return objects



# -*- coding: utf-8 -*-

"""
This file is part of the Ingram Micro Cloud Blue Connect SDK.
Copyright (c) 2019 Ingram Micro. All Rights Reserved.
"""
from abc import ABCMeta

from connect.logger import logger
from connect.models.exception import UsageFileAction, Skip
from connect.models.usage import FileSchema, File
from connect.resource import AutomationResource


class UsageFileAutomation(AutomationResource):
    __metaclass__ = ABCMeta
    resource = 'usage/files'
    schema = FileSchema(many=True)

    def dispatch(self, request):
        # type: (File) -> str
        try:
            # Validate product
            if self.config.products \
                    and request.product.id not in self.config.products:
                return 'Invalid product'

            # Process request
            logger.info(
                'Start usage file request process / ID request - {}'.format(request.id))
            result = self.process_request(request)

            # Report that expected exception was not raised
            processing_result = 'UsageFileAutomation.process_request returned {} ' \
                                'while is expected to raise UsageFileAction or Skip exception' \
                .format(str(result))
            logger.warning(processing_result)
            raise UserWarning(processing_result)

        # Catch action
        except UsageFileAction as usage:
            self.api.post(path='{}/{}'.format(request.id, usage.code), data=usage.obj)
            processing_result = usage.code

        # Catch skip
        except Skip:
            processing_result = 'skip'

        logger.info('Finished processing of usage file with ID {} with result {}'
                    .format(request.id, processing_result))
        return processing_result



# -*- coding: utf-8 -*-

"""
This file is part of the Ingram Micro Cloud Blue Connect SDK.
Copyright (c) 2019 Ingram Micro. All Rights Reserved.
"""

from marshmallow import Schema, fields, post_load

from .base import BaseModel, BaseSchema
from .contact import ContactInfo, ContactInfoSchema


class Tier(BaseModel):
    name = None  # type: str
    contact_info = None  # type: ContactInfo
    external_id = None  # type: str
    external_uid = None  # type: str


class TierSchema(BaseSchema):
    name = fields.Str()
    contact_info = fields.Nested(ContactInfoSchema)
    external_id = fields.Str()
    external_uid = fields.UUID()

    @post_load
    def make_object(self, data):
        return Tier(**data)


class Tiers(BaseModel):
    customer = None  # type: Tier
    tier1 = None  # type: Tier
    tier2 = None  # type: Tier


class TiersSchema(Schema):
    customer = fields.Nested(TierSchema)
    tier1 = fields.Nested(TierSchema)
    tier2 = fields.Nested(TierSchema)

    @post_load
    def make_object(self, data):
        return Tiers(**data)



# -*- coding: utf-8 -*-

"""
This file is part of the Ingram Micro Cloud Blue Connect SDK.
Copyright (c) 2019 Ingram Micro. All Rights Reserved.
"""

from marshmallow import Schema, fields, post_load
from typing import List

from .base import BaseModel


class ServerError(BaseModel):
    error_code = None  # type: str
    params = None  # type: dict
    errors = None  # type: List[str]


class ServerErrorSchema(Schema):
    error_code = fields.Str()
    params = fields.Dict()
    errors = fields.List(fields.Str())

    @post_load
    def make_object(self, data):
        return ServerError(**data)



# -*- coding: utf-8 -*-

"""
This file is part of the Ingram Micro Cloud Blue Connect SDK.
Copyright (c) 2019 Ingram Micro. All Rights Reserved.
"""

from marshmallow import fields, post_load

from .base import BaseModel, BaseSchema


class PhoneNumber(BaseModel):
    country_code = None  # type: str
    area_code = None  # type: str
    phone_number = None  # type: str
    extension = None  # type: str


class PhoneNumberSchema(BaseSchema):
    country_code = fields.Str()
    area_code = fields.Str()
    phone_number = fields.Str()
    extension = fields.Str()

    @post_load
    def make_object(self, data):
        return PhoneNumber(**data)


class Contact(BaseModel):
    email = None  # type: str
    first_name = None  # type: str
    last_name = None  # type: str
    phone_number = None  # type: PhoneNumber


class ContactSchema(BaseSchema):
    email = fields.Str()
    first_name = fields.Str()
    last_name = fields.Str()
    phone_number = fields.Nested(PhoneNumberSchema)

    @post_load
    def make_object(self, data):
        return Contact(**data)


class ContactInfo(BaseModel):
    address_line1 = None  # type: str
    address_line2 = None  # type: str
    city = None  # type: str
    contact = None  # type: Contact
    country = None  # type: str
    postal_code = None  # type: str
    state = None  # type: str


class ContactInfoSchema(BaseSchema):
    address_line1 = fields.Str()
    address_line2 = fields.Str()
    city = fields.Str()
    contact = fields.Nested(ContactSchema)
    country = fields.Str()
    postal_code = fields.Str()
    state = fields.Str()

    @post_load
    def make_object(self, data):
        return ContactInfo(**data)



# -*- coding: utf-8 -*-

"""
This file is part of the Ingram Micro Cloud Blue Connect SDK.
Copyright (c) 2019 Ingram Micro. All Rights Reserved.
"""
from typing import List, Dict, Any, Optional

from connect.models import Param
from .server_error import ServerError


class Message(Exception):
    code = None  # type: str
    obj = None  # type: object

    def __init__(self, message='', code='', obj=None):
        # type: (str, str, object) -> None
        self.message = message
        self.code = code
        self.obj = obj


class FulfillmentFail(Message):
    def __init__(self, *args, **kwargs):
        # type: (*any, **any) -> None
        super(FulfillmentFail, self).__init__(*args, **kwargs)
        self.message = self.message or 'Request failed'
        self.code = 'fail'


class FulfillmentInquire(Message):
    params = None  # type: List[Param]

    def __init__(self, *args, **kwargs):
        # type: (*any, **any) -> None
        super(FulfillmentInquire, self).__init__(*args, **kwargs)
        self.message = self.message or 'Correct user input required'
        self.code = 'inquire'
        self.params = kwargs.get('params', [])


class Skip(Message):
    def __init__(self, *args, **kwargs):
        # type: (*any, **any) -> None
        super(Skip, self).__init__(*args, **kwargs)
        self.message = self.message or 'Request skipped'
        self.code = 'skip'


class UsageFileAction(Message):
    def __init__(self, message, code, data=None):
        # type: (str, str, Optional[Dict[str, Any]]) -> None
        super(UsageFileAction, self).__init__(message, code, data)


class AcceptUsageFile(UsageFileAction):
    def __init__(self, acceptance_note):
        # type: (str) -> None
        super(AcceptUsageFile, self).__init__(
            'Accept Response is required',
            'accept',
            {'acceptance_note': acceptance_note})


class CloseUsageFile(UsageFileAction):
    def __init__(self, message=None):
        # type: (str) -> None
        super(CloseUsageFile, self).__init__(message or 'Usage File Closed', 'close')


class DeleteUsageFile(UsageFileAction):
    def __init__(self, message=None):
        # type: (str) -> None
        super(DeleteUsageFile, self).__init__(message or 'Usage File Deleted', 'delete')


class RejectUsageFile(UsageFileAction):
    def __init__(self, message=None):
        # type: (str) -> None
        super(RejectUsageFile, self).__init__(message or 'Accept Response is required', 'reject')


class SubmitUsageFile(UsageFileAction):
    def __init__(self, rejection_note):
        # type: (str) -> None
        super(SubmitUsageFile, self).__init__(
            'Usage File Submited',
            'submit',
            {'rejection_note': rejection_note})


class ServerErrorException(Exception):
    message = 'Server error'  # type: str

    def __init__(self, error=None, *args, **kwargs):
        # type: (ServerError, *any, **any) -> None

        if error and isinstance(error, ServerError):
            self.message = str({
                "error_code": error.error_code,
                "params": kwargs.get('params', []),
                "errors": error.errors,
            })
        else:
            self.message = self.__class__.message

        super(ServerErrorException, self).__init__(self.message, *args)


class FileCreationError(Message):
    def __init__(self, message):
        # type: (str) -> None
        super(FileCreationError, self).__init__(message, 'filecreation')


class FileRetrievalError(Message):
    def __init__(self, message):
        # type: (str) -> None
        super(FileRetrievalError, self).__init__(message, 'fileretrieval')



# -*- coding: utf-8 -*-

"""
This file is part of the Ingram Micro Cloud Blue Connect SDK.
Copyright (c) 2019 Ingram Micro. All Rights Reserved.
"""

from marshmallow import fields, post_load

from .base import BaseModel, BaseSchema
from .company import Company, CompanySchema
from .hub import Hub, HubSchema
from .product import Product, ProductSchema


class Connection(BaseModel):
    type = None  # type: str
    provider = None  # type: Company
    vendor = None  # type: Company
    product = None  # type: Product
    hub = None  # type: Hub


class ConnectionSchema(BaseSchema):
    type = fields.Str()
    provider = fields.Nested(CompanySchema, only=('id', 'name'))
    vendor = fields.Nested(CompanySchema, only=('id', 'name'))
    product = fields.Nested(ProductSchema)
    hub = fields.Nested(HubSchema)

    @post_load
    def make_object(self, data):
        return Connection(**data)



# -*- coding: utf-8 -*-

"""
This file is part of the Ingram Micro Cloud Blue Connect SDK.
Copyright (c) 2019 Ingram Micro. All Rights Reserved.
"""
from typing import List, Any

from connect.models import ActivationTemplateResponse, ActivationTileResponse
from .base import BaseResource


class TemplateResource(BaseResource):
    """ Specific resource.
    One method `render`
    He returns json string with activation tile
    """
    resource = 'templates'

    @property
    def list(self):
        # type: () -> List[Any]
        raise AttributeError('This resource do not have method `list`')

    def render(self, pk, request_id):
        # type: (str, str) -> ActivationTileResponse
        if not all([pk, request_id]):
            raise ValueError('Invalid ids for render template')
        response = self.api.get(path=pk + '/render', params={'request_id': request_id})
        return ActivationTileResponse(response)

    def get(self, pk):
        # type: (str) -> ActivationTemplateResponse
        return ActivationTemplateResponse(template_id=pk)



# -*- coding: utf-8 -*-

"""
This file is part of the Ingram Micro Cloud Blue Connect SDK.
Copyright (c) 2019 Ingram Micro. All Rights Reserved.
"""

from marshmallow import Schema, fields, post_load
from typing import List

from .base import BaseModel, BaseSchema


class ValueChoice(BaseModel):
    value = None  # type: str
    label = None  # type: str


class Constraints(BaseModel):
    hidden = None  # type: bool
    required = None  # type: bool
    choices = None  # type: List[ValueChoice]


class Param(BaseModel):
    name = None  # type: str
    type = None  # type: str
    value = None  # type: str
    value_choices = None  # type: List[ValueChoice]
    value_error = None  # type: str

    # Undocumented fields (they appear in PHP SDK)
    title = None  # type: str
    scope = None  # type: str
    constraints = None  # type: Constraints


class ValueChoiceSchema(Schema):
    value = fields.Str()
    label = fields.Str()

    @post_load
    def make_object(self, data):
        return ValueChoice(**data)


class ConstraintsSchema(BaseSchema):
    hidden = fields.Bool()
    required = fields.Bool()
    choices = fields.List(fields.Nested(ValueChoiceSchema))

    @post_load
    def make_object(self, data):
        return Constraints(**data)


class ParamSchema(BaseSchema):
    name = fields.Str()
    type = fields.Str()
    value = fields.Str()
    value_choices = fields.List(fields.Nested(ValueChoiceSchema))
    value_error = fields.Str()

    # Undocumented fields (they appear in PHP SDK)
    title = fields.Str(required=False)
    scope = fields.Str(required=False)
    constraints = fields.Nested(ConstraintsSchema, required=False)

    @post_load
    def make_object(self, data):
        return Param(**data)



# -*- coding: utf-8 -*-

"""
This file is part of the Ingram Micro Cloud Blue Connect SDK.
Copyright (c) 2019 Ingram Micro. All Rights Reserved.
"""
from abc import ABCMeta

from typing import Optional

from connect.logger import logger
from connect.models import ActivationTemplateResponse, ActivationTileResponse
from connect.models.exception import FulfillmentFail, FulfillmentInquire, Skip
from connect.models.tier_config import TierConfigRequest, TierConfigRequestSchema, TierConfig
from .automation import AutomationResource


class TierConfigAutomation(AutomationResource):
    __metaclass__ = ABCMeta
    resource = 'tier/config-requests'
    schema = TierConfigRequestSchema(many=True)

    def dispatch(self, request):
        # type: (TierConfigRequest) -> str
        try:
            if self.config.products \
                    and request.configuration.product.id not in self.config.products:
                return 'Invalid product'

            logger.info(
                'Start tier config request process / ID request - {}'.format(request.id))
            result = self.process_request(request)

            if not result:
                logger.info('Method `process_request` did not return result')
                return ''

            params = {}
            if isinstance(result, ActivationTileResponse):
                params = {'template': {'representation': result.tile}}
            elif isinstance(result, ActivationTemplateResponse):
                params = {'template': {'id': result.template_id}}

            self.approve(request.id, params)

        except FulfillmentInquire as inquire:
            self.update_parameters(request.id, inquire.params)
            return self.inquire(request.id)

        except FulfillmentFail as fail:
            return self.fail(request.id, reason=fail.message)

        except Skip as skip:
            return skip.code

        return ''

    def get_tier_config(self, tier_id, product_id):
        # type: (str, str) -> Optional[TierConfig]
        params = {
            'status': 'approved',
            'configuration__product__id': product_id,
            'configuration__account__id': tier_id,
        }
        response = self.api.get(params=params)
        objects = self._load_schema(response)

        if isinstance(objects, list) and len(objects) > 0:
            return objects[0].configuration
        else:
            return None



# -*- coding: utf-8 -*-

"""
This file is part of the Ingram Micro Cloud Blue Connect SDK.
Copyright (c) 2019 Ingram Micro. All Rights Reserved.
"""

from .activation_response import ActivationTemplateResponse, ActivationTileResponse
from .base import BaseSchema
from .fulfillment import FulfillmentSchema
from .parameters import Param, ParamSchema
from .product import Item, ItemSchema
from .server_error import ServerErrorSchema

__all__ = [
    'ActivationTemplateResponse',
    'ActivationTileResponse',
    'BaseSchema',
    'FulfillmentSchema',
    'Item',
    'ItemSchema',
    'Param',
    'ParamSchema',
    'ServerErrorSchema',
]



# -*- coding: utf-8 -*-

"""
This file is part of the Ingram Micro Cloud Blue Connect SDK.
Copyright (c) 2019 Ingram Micro. All Rights Reserved.
"""

import json
import logging
import os
from logging.config import dictConfig

with open(os.path.join(os.path.dirname(__file__), 'config.json')) as config_file:
    config = json.load(config_file)

dictConfig(config['logging'])
logger = logging.getLogger()


def function_log(func):
    def decorator(self, *args, **kwargs):
        logger.info('Entering: %s', func.__name__)
        logger.debug('Function params: {} {}'.format(args, kwargs))

        result = func(self, *args, **kwargs)

        logger.debug(
            'Function `{}.{}` return: {}'.format(self.__class__.__name__, func.__name__, result))
        return result

    return decorator



# -*- coding: utf-8 -*-

"""
This file is part of the Ingram Micro Cloud Blue Connect SDK.
Copyright (c) 2019 Ingram Micro. All Rights Reserved.
"""

import json
import os

from typing import List, Union


class Config(object):
    # Global instance
    _instance = None  # type: Config

    # noinspection PyShadowingBuiltins
    def __init__(
            self,
            api_url=None,  # type: str
            api_key=None,  # type: str
            products=None,  # type: Union[str, List[str]]
            file=None  # type: str
    ):
        """
        Initialization config for public api
        :param api_url: Public api url
        :param api_key: Service user ApiKey
        :param products (optional): Id products
        :param file: Config file name
        """

        # Check arguments
        if not file and not any([api_key, api_url]):
            raise ValueError('Filename or api_key and api_url are expected'
                             'in Config initialization')
        if products and not isinstance(products, (str, list)):
            raise TypeError('Products can be string or string list. Found type '
                            + type(products).__name__)

        # Load config from file name
        if file:
            if not os.path.exists(file):
                raise IOError('No file `{}` on directory'.format(file))

            with open(file) as config_file:
                configs = config_file.read()

            try:
                configs = json.loads(configs)
            except Exception as ex:
                raise TypeError('Invalid config file `{}`\n'
                                'ERROR: {}'.format(file, str(ex)))

            (api_url, api_key, products) = (configs.get('apiEndpoint', ''),
                                            configs.get('apiKey', ''),
                                            configs.get('products', ''))
            api_url = api_url.encode('utf-8') if not isinstance(api_url, str) else api_url
            api_key = api_key.encode('utf-8') if not isinstance(api_key, str) else api_key
            products = products.encode('utf-8') \
                if not isinstance(products, (str, list)) \
                else products

        # Initialize
        self._api_key = api_key
        self._api_url = api_url
        self._products = [products] \
            if isinstance(products, str) \
            else products or []

        # Store first created instance
        if not Config._instance:
            Config._instance = self

    @classmethod
    def get_instance(cls):
        # type: () -> Config
        if not cls._instance:
            cls._instance = Config(file='config.json')
        return cls._instance

    @property
    def api_url(self):
        # type: () -> str
        return self._api_url

    @property
    def api_key(self):
        # type: () -> str
        return self._api_key

    @property
    def products(self):
        # type: () -> List[str]
        return self._products



# -*- coding: utf-8 -*-

"""
This file is part of the Ingram Micro Cloud Blue Connect SDK.
Copyright (c) 2019 Ingram Micro. All Rights Reserved.
"""

import json


class ActivationTileResponse(object):
    tile = 'Activation succeeded'  # type: str

    def __init__(self, markdown=''):
        # type: (str) -> None
        try:
            self.tile = json.loads(markdown)
        except ValueError:
            self.tile = markdown or self.__class__.tile


class ActivationTemplateResponse(object):
    template_id = None  # type: str

    def __init__(self, template_id):
        # type: (str) -> None
        self.template_id = template_id



# -*- coding: utf-8 -*-

"""
This file is part of the Ingram Micro Cloud Blue Connect SDK.
Copyright (c) 2019 Ingram Micro. All Rights Reserved.
"""

from marshmallow import fields, post_load

from .base import BaseModel, BaseSchema
from .company import Company, CompanySchema
from .hub import Hubs, HubsSchema


class Marketplace(BaseModel):
    name = None  # type: str
    zone = None  # type: str
    description = None  # type: str
    active_contract = None  # type: int
    icon = None  # type: str
    owner = None  # type: Company
    hubs = None  # type: Hubs


class MarketplaceSchema(BaseSchema):
    name = fields.Str()
    zone = fields.Str()
    description = fields.Str()
    active_contract = fields.Int()
    icon = fields.Str()
    owner = fields.Nested(CompanySchema, only=('id', 'name'))
    hubs = fields.List(fields.Nested(HubsSchema, only=('id', 'name')))

    @post_load
    def make_object(self, data):
        return Marketplace(**data)


class Agreement(BaseModel):
    type = None  # type: str
    title = None  # type: str
    description = None  # type: str
    created = None  # type: str
    updated = None  # type: str
    owner = None  # type: Company
    stats = None  # type: dict
    active = None  # type: bool
    version = None  # type: int
    link = None  # type: str
    version_created = None  # type: str
    version_contracts = None  # type: int


class AgreementSchema(BaseSchema):
    type = fields.Str()
    title = fields.Str()
    description = fields.Str()
    created = fields.DateTime()
    updated = fields.DateTime()
    owner = fields.Nested(CompanySchema, only=('id', 'name'))
    stats = fields.Dict()
    active = fields.Bool()
    version = fields.Int()
    link = fields.Str()
    version_created = fields.DateTime()
    version_contracts = fields.Int()

    @post_load
    def make_object(self, data):
        return Agreement(**data)


class Contract(BaseModel):
    name = None  # type: str
    status = None  # type: str
    version = None  # type: int
    type = None  # type: str
    agreement = None  # type: Agreement
    marketplace = None  # type: Marketplace
    owner = None  # type: Company
    created = None  # type: str
    updated = None  # type: str
    enrolled = None  # type: str
    version_created = None  # type: str
    activation = None  # type: dict
    signee = None  # type: Company


class ContractSchema(BaseSchema):
    name = fields.Str()
    status = fields.Str()
    version = fields.Int()
    type = fields.Str()
    agreement = fields.Nested(AgreementSchema, only=('id', 'name'))
    marketplace = fields.Nested(MarketplaceSchema, only=('id', 'name'))
    owner = fields.Nested(CompanySchema, only=('id', 'name'))
    created = fields.DateTime()
    updated = fields.DateTime()
    enrolled = fields.Str()
    version_created = fields.DateTime()
    activation = fields.Dict()
    signee = fields.Nested(CompanySchema, only=('id', 'name'))

    @post_load
    def make_object(self, data):
        return Contract(**data)



# -*- coding: utf-8 -*-

"""
This file is part of the Ingram Micro Cloud Blue Connect SDK.
Copyright (c) 2019 Ingram Micro. All Rights Reserved.
"""
from marshmallow import fields, post_load
from typing import Optional, List

from connect.models import Param, ParamSchema
from connect.models.base import BaseModel, BaseSchema
from connect.models.company import Company, CompanySchema
from connect.models.connection import Connection, ConnectionSchema
from connect.models.contact import ContactInfo, ContactInfoSchema
from connect.models.product import Product, ProductSchema


class Account(BaseModel):
    name = None  # type: str
    external_id = None  # type: str
    external_uid = None  # type: str
    contact_info = None  # type: ContactInfo


class EventInfo(BaseModel):
    at = None  # type: str
    by = None  # type: Company


class Events(BaseModel):
    created = None  # type: EventInfo
    inquired = None  # type: EventInfo
    pended = None  # type: EventInfo
    validated = None  # type: EventInfo
    updated = None  # type: EventInfo


class Template(BaseModel):
    representation = None  # type: str


class TierConfig(BaseModel):
    name = None  # type: str
    account = None  # type: Account
    product = None  # type: Product
    tier_level = None  # type: int
    connection = None  # type: Connection
    events = None  # type: Events
    params = None  # type: List[Param]
    template = None  # type: Template

    # Undocumented fields (they appear in PHP SDK)
    open_request = None  # type: BaseModel

    def get_param_by_id(self, id_):
        # type: (str) -> Optional[Param]
        try:
            return list(filter(lambda param: param.id == id_, self.params))[0]
        except IndexError:
            return None


class Activation(BaseModel):
    link = None  # type: str


class TierConfigRequest(BaseModel):
    type = None  # type: str
    status = None  # type: str
    configuration = None  # type: TierConfig
    events = None  # type: Events
    params = None  # type: List[Param]
    assignee = None  # type: Company
    template = None  # type: Template
    activation = None  # type: Activation

    def get_param_by_id(self, id_):
        # type: (str) -> Optional[Param]
        try:
            return list(filter(lambda param: param.id == id_, self.params))[0]
        except IndexError:
            return None


class AccountSchema(BaseSchema):
    name = fields.Str()
    external_id = fields.Str()
    external_uid = fields.Str()
    contact_info = fields.Nested(ContactInfoSchema)

    @post_load
    def make_object(self, data):
        return Account(**data)


class EventInfoSchema(BaseSchema):
    at = fields.Str()
    by = fields.Nested(CompanySchema)

    @post_load
    def make_object(self, data):
        return EventInfo(**data)


class EventsSchema(BaseSchema):
    created = fields.Nested(EventInfoSchema)
    inquired = fields.Nested(EventInfoSchema, required=False)
    pended = fields.Nested(EventInfoSchema, required=False)
    validated = fields.Nested(EventInfoSchema, required=False)
    updated = fields.Nested(EventInfoSchema, required=False)

    @post_load
    def make_object(self, data):
        return Events(**data)


class TemplateSchema(BaseSchema):
    representation = fields.Str()

    @post_load
    def make_object(self, data):
        return Template(**data)


class TierConfigSchema(BaseSchema):
    name = fields.Str()
    account = fields.Nested(AccountSchema)
    product = fields.Nested(ProductSchema)
    tier_level = fields.Int()
    connection = fields.Nested(ConnectionSchema)
    events = fields.Nested(EventsSchema)
    params = fields.List(fields.Nested(ParamSchema))
    template = fields.Nested(TemplateSchema)

    # Undocumented fields (they appear in PHP SDK)
    open_request = fields.Nested(BaseSchema)

    @post_load
    def make_object(self, data):
        return TierConfig(**data)


class ActivationSchema(BaseSchema):
    link = fields.Str()

    @post_load
    def make_object(self, data):
        return Activation(**data)


class TierConfigRequestSchema(BaseSchema):
    type = fields.Str()
    status = fields.Str()
    configuration = fields.Nested(TierConfigSchema)
    events = fields.Nested(EventsSchema)
    params = fields.List(fields.Nested(ParamSchema))
    assignee = fields.Nested(CompanySchema)
    template = fields.Nested(TemplateSchema)
    activation = fields.Nested(ActivationSchema)

    @post_load
    def make_object(self, data):
        return TierConfigRequest(**data)



# -*- coding: utf-8 -*-

"""
This file is part of the Ingram Micro Cloud Blue Connect SDK.
Copyright (c) 2019 Ingram Micro. All Rights Reserved.
"""

from .logger import function_log, logger

# TODO add auto settings for cloud platforms

__all__ = [
    'function_log',
    'logger'
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
This file is part of the Ingram Micro Cloud Blue Connect SDK.
Copyright (c) 2019 Ingram Micro. All Rights Reserved.
"""

from os.path import abspath, dirname, exists, join
from os import environ
from setuptools import find_packages, setup

try:  # for pip >= 10
    # noinspection PyProtectedMember,PyPackageRequirements
    from pip._internal.req import parse_requirements
except ImportError:  # for pip <= 9.0.3
    # noinspection PyPackageRequirements,PyUnresolvedReferences
    from pip.req import parse_requirements

# noinspection PyTypeChecker
install_reqs = parse_requirements(
    join(
        dirname(abspath(__file__)),
        'requirements',
        'sdk.txt',
    ), session='None')

VERSION = environ.get('TRAVIS_TAG')
PACKAGES = find_packages(exclude=['tests*'])

DOC = ''
if exists('README.md'):
    DOC = open('README.md', 'r').read()

setup(
    name='connect-sdk',
    author='Ingram Micro',
    version=VERSION,
    keywords='sdk connect connect automation',
    packages=PACKAGES,
    description='Connect Python SDK',
    long_description=DOC,
    long_description_content_type='text/markdown',
    url='https://github.com/ingrammicro/connect-python-sdk',
    license='Apache Software License',
    include_package_data=True,
    install_requires=[str(ir.req) for ir in install_reqs],

    classifiers=[
        'Development Status :: 4 - Beta',

        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',

        'License :: OSI Approved :: Apache Software License',

        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',

        'Operating System :: OS Independent',
        'Operating System :: POSIX',
        'Operating System :: MacOS',
        'Operating System :: Unix',
    ],
)



# -*- coding: utf-8 -*-

"""
This file is part of the Ingram Micro Cloud Blue Connect SDK.
Copyright (c) 2019 Ingram Micro. All Rights Reserved.
"""
from abc import ABCMeta

from typing import Any, Dict

from connect.logger import logger
from connect.models import ActivationTemplateResponse, ActivationTileResponse
from connect.models.exception import FulfillmentFail, FulfillmentInquire, Skip
from connect.models.fulfillment import Fulfillment, FulfillmentSchema
from .automation import AutomationResource


class FulfillmentAutomation(AutomationResource):
    __metaclass__ = ABCMeta
    resource = 'requests'
    schema = FulfillmentSchema(many=True)

    def build_filter(self, status='pending'):
        # type: (str) -> Dict[str, Any]
        filters = super(FulfillmentAutomation, self).build_filter(status)
        if self.config.products:
            filters['asset.product.id__in'] = ','.join(self.config.products)
        return filters

    def dispatch(self, request):
        # type: (Fulfillment) -> str
        try:
            if self.config.products \
                    and request.asset.product.id not in self.config.products:
                return 'Invalid product'

            logger.info('Start request process / ID request - {}'.format(request.id))
            result = self.process_request(request)

            if not result:
                logger.info('Method `process_request` did not return result')
                return ''

            params = {}
            if isinstance(result, ActivationTileResponse):
                params = {'activation_tile': result.tile}
            elif isinstance(result, ActivationTemplateResponse):
                params = {'template_id': result.template_id}

            return self.approve(request.id, params)

        except FulfillmentInquire as inquire:
            self.update_parameters(request.id, inquire.params)
            return self.inquire(request.id)

        except FulfillmentFail as fail:
            return self.fail(request.id, reason=fail.message)

        except Skip as skip:
            return skip.code



# -*- coding: utf-8 -*-

"""
This file is part of the Ingram Micro Cloud Blue Connect SDK.
Copyright (c) 2019 Ingram Micro. All Rights Reserved.
"""

from marshmallow import Schema, fields, post_load


class BaseModel:
    id = None  # type: str

    def __init__(self, **kwargs):
        # Inject parsed properties in the model
        for attr, val in kwargs.items():
            setattr(self, attr, val)


class BaseSchema(Schema):
    id = fields.Str()

    @post_load
    def make_object(self, data):
        return BaseModel(**data)



# -*- coding: utf-8 -*-

"""
This file is part of the Ingram Micro Cloud Blue Connect SDK.
Copyright (c) 2019 Ingram Micro. All Rights Reserved.
"""

from .resource import FulfillmentAutomation, TierConfigAutomation

name = 'connect'

__all__ = [
    'config',
    'resource',
    'models',
    'logger',
    'FulfillmentAutomation',
    'TierConfigAutomation',
]



# -*- coding: utf-8 -*-

"""
This file is part of the Ingram Micro Cloud Blue Connect SDK.
Copyright (c) 2019 Ingram Micro. All Rights Reserved.
"""
import json
from abc import ABCMeta
from tempfile import NamedTemporaryFile

import openpyxl
import requests
from typing import Dict, Any, List, Optional

from connect.logger import logger
from connect.models.exception import FileCreationError, FileRetrievalError
from connect.models.product import Product
from connect.models.usage import FileSchema, Listing, File, FileUsageRecord
from connect.resource import AutomationResource


class UsageAutomation(AutomationResource):
    __metaclass__ = ABCMeta
    resource = 'usage/files'
    schema = FileSchema(many=True)

    def build_filter(self, status='listed'):
        # type: (str) -> Dict[str, Any]
        filters = super(UsageAutomation, self).build_filter(status)
        if self.config.products:
            filters['product__id'] = ','.join(self.config.products)
        return filters

    def dispatch(self, request):
        # type: (Listing) -> str

        # TODO Shouldn't this raise an exception on ALL automation classes?
        if self.config.products \
                and request.product.id not in self.config.products:
            return 'Listing not handled by this processor'

        logger.info(
            'Processing Usage for Product {} ({}) '.format(request.product.id,
                                                           request.product.name) +
            'on Contract {} '.format(request.contract.id) +
            'and provider {}({})'.format(request.provider.id, request.provider.name))

        try:
            result = self.process_request(request)
        except FileCreationError:
            logger.info(
                'Error processing Usage for Product {} ({}) '.format(request.product.id,
                                                                     request.product.name) +
                'on Contract {} '.format(request.contract.id) +
                'and provider {}({})'.format(request.provider.id, request.provider.name))
            return 'failure'

        logger.info('Processing result for usage on listing {}: {}'
                    .format(request.product.id, result))
        return 'success'

    def create_usage_file(self, usage_file):
        # type: (File) -> File
        if not usage_file.name or not usage_file.product.id or not usage_file.contract.id:
            raise FileCreationError('Usage File Creation requires name, product id, contract id')
        if not usage_file.description:
            # Could be because description is empty or None, so make sure it is empty
            usage_file.description = ''
        response = self.api.post(data=usage_file)
        return self._load_schema(response, many=False)

    @staticmethod
    def create_spreadsheet(usage_records):
        # type: (List[FileUsageRecord]) -> openpyxl.Workbook
        book = openpyxl.Workbook()
        sheet = book.active
        sheet.title = 'usage_records'
        sheet['A1'] = 'record_id'
        sheet['B1'] = 'item_search_criteria'
        sheet['C1'] = 'item_search_value'
        sheet['D1'] = 'quantity'
        sheet['E1'] = 'start_time_utc'
        sheet['F1'] = 'end_time_utc'
        sheet['G1'] = 'asset_search_criteria'
        sheet['H1'] = 'asset_search_value'
        for index, record in enumerate(usage_records):
            row = str(index + 2)
            sheet['A' + row] = record.record_id
            sheet['B' + row] = record.item_search_criteria
            sheet['C' + row] = record.item_search_value
            sheet['D' + row] = record.quantity
            sheet['E' + row] = record.start_time_utc
            sheet['F' + row] = record.end_time_utc
            sheet['G' + row] = record.asset_search_criteria
            sheet['H' + row] = record.asset_search_value
        return book

    def upload_spreadsheet(self, usage_file, spreadsheet):
        # type: (File, openpyxl.Workbook) -> None

        # Generate spreadsheet file
        with NamedTemporaryFile() as tmp:
            spreadsheet.save(tmp)
            tmp.seek(0)
            file_contents = tmp.read()

        # Setup request
        url = self.api.get_url(usage_file.id + '/upload/')
        headers = self.api.headers
        headers['Accept'] = 'application/json'
        del headers['Content-Type']  # This must NOT be set for multipart post requests
        multipart = {'usage_file': ('usage_file.xlsx', file_contents)}
        logger.info('HTTP Request: {} - {} - {}'.format(url, headers, multipart))

        # Post request
        try:
            response = requests.post(url, headers=headers, files=multipart)
        except requests.RequestException as ex:
            raise FileCreationError('Error uploading file: {}'.format(ex))
        logger.info('HTTP Code: {}'.format(response.status_code))
        if response.status_code != 201:
            msg = 'Unexpected server response, returned code {}'.format(response.status_code)
            logger.error('{} -- Raw response: {}'.format(msg, response.content))
            raise FileCreationError(msg)

    def upload_usage_records(self, usage_file, usage_records):
        # type: (File, List[FileUsageRecord]) -> None
        # TODO: Using xslx mechanism till usage records json api is available
        book = self.create_spreadsheet(usage_records)
        self.upload_spreadsheet(usage_file, book)

    def get_usage_template(self, product):
        # type: (Product) -> bytes
        location = self._get_usage_template_download_location(product.id)
        if not location:
            msg = 'Error obtaining template usage file location'
            logger.error(msg)
            raise FileRetrievalError(msg)

        contents = self._retrieve_usage_template(location) if location else None
        if not contents:
            msg = 'Error obtaining template usage file from `{}`'.format(location)
            logger.error(msg)
            raise FileRetrievalError(msg)
        return contents

    def submit_usage(self, usage_file, usage_records):
        # type: (File, List[FileUsageRecord]) -> File
        usage_file = self.create_usage_file(usage_file)
        self.upload_usage_records(usage_file, usage_records)
        return usage_file

    def _get_usage_template_download_location(self, product_id):
        # type: (str) -> str
        try:
            response = self.api.get(url='{}/usage/products/{}/template/'
                                    .format(self.config.api_url, product_id))
            response_dict = json.loads(response)
            return response_dict['template_link']
        except (requests.exceptions.RequestException, KeyError, TypeError, ValueError):
            return ''

    @staticmethod
    def _retrieve_usage_template(location):
        # type: (str) -> Optional[bytes]
        try:
            response = requests.get(location)
            return response.content
        except requests.exceptions.RequestException:
            return None



# -*- coding: utf-8 -*-

"""
This file is part of the Ingram Micro Cloud Blue Connect SDK.
Copyright (c) 2019 Ingram Micro. All Rights Reserved.
"""

import json

from typing import Any, List, Dict

from connect.logger import function_log
from connect.models import Param, ActivationTileResponse
from connect.models.base import BaseModel
from .base import BaseResource
from .template import TemplateResource


class AutomationResource(BaseResource):
    limit = 1000

    def build_filter(self, status='pending'):
        # type: (str) -> Dict[str, Any]
        filters = super(AutomationResource, self).build_filter()
        if status:
            filters['status'] = status
        return filters

    def process(self):
        # type: () -> None
        for request in self.list:
            self.dispatch(request)

    def dispatch(self, request):
        # type: (BaseModel) -> str
        raise NotImplementedError('Please implement `{}.dispatch` method'
                                  .format(self.__class__.__name__))

    def process_request(self, request):
        raise NotImplementedError('Please implement `{}.process_request` method'
                                  .format(self.__class__.__name__))

    @function_log
    def approve(self, pk, data):
        # type: (str, dict) -> str
        return self.api.post(path=pk + '/approve/', data=data if data else {})

    @function_log
    def inquire(self, pk):
        # type: (str) -> str
        return self.api.post(path=pk + '/inquire/', data={})

    @function_log
    def fail(self, pk, reason):
        # type: (str, str) -> str
        return self.api.post(path=pk + '/fail/', data={'reason': reason})

    @function_log
    def render_template(self, pk, template_id):
        # type: (str, str) -> ActivationTileResponse
        return TemplateResource(self.config).render(template_id, pk)

    @function_log
    def update_parameters(self, pk, params):
        # type: (str, List[Param]) -> str
        list_dict = []
        for _ in params:
            list_dict.append(_.__dict__ if isinstance(_, Param) else _)

        return self.api.put(
            path=pk,
            data=json.dumps({'asset': {'params': list_dict}}),
        )
